#include<stdio.h>
#include<stdlib.h>
#include"config.h"
#include"util.h"
#include"dico.h"
#include"feat_types.h"

config *config_new(FILE *f, mcd *mcd_struct, int lookahead)
{
  config *c = (config *)memalloc(sizeof(config));
  c->ds = depset_new();
  c->f = f;
  c->mcd_struct = mcd_struct;
  c->st = stack_new();
  c->bf = word_buffer_new(f, mcd_struct, lookahead);
  c->score = 0;
  c->mvt_array = NULL;
  c->mvt_array_size = 0;
  c->mvt_array_nbelem = 0;
  c->gfv = global_feat_vec_new();

  c->current_index = 1;
  return c;
}

/*word *config_add_next_word_to_buffer(config *c)
{
  word *w = NULL;

  w = word_read(c->f, c->mcd_struct);
  if(w == NULL) return NULL;

  word_set_index(w, c->current_index);
  c->current_index++;
  queue_add(c->bf, w);
  return w;
  }*/

void config_free(config *c)
{
  stack_free(c->st);
  word_buffer_free(c->bf);
  depset_free(c->ds);
  if(c->mvt_array)free(c->mvt_array);
  if(c->gfv)global_feat_vec_free(c->gfv);
  free(c);
}

int config_is_terminal(config *c)
{
  return word_buffer_end(c->bf);
}

config *config_initial(FILE *f, mcd *mcd_struct, int lookahead)
{
  config *c = config_new(f, mcd_struct, lookahead);

  stack_push(c->st, word_create_dummy(mcd_struct));

  return c;
}

config *config_initial_no_dummy_word(FILE *f, mcd *mcd_struct, int lookahead)
{
  config *c = config_new(f, mcd_struct, lookahead);

  return c;
}

/* must be checked things probably missing */

config *config_copy(config *o) 
{
  int i;
  config *c = (config *)memalloc(sizeof(config));
  c->ds = depset_copy(o->ds);
  c->st = stack_copy(o->st);
  /* c->bf = queue_copy(o->bf); */
  c->score = o->score;

  c->f = o->f;
  c->mcd_struct = o->mcd_struct;

  c->mvt_array_size = o->mvt_array_size;
  c->mvt_array_nbelem = o->mvt_array_nbelem;
  c->mvt_array = (int *)memalloc(c->mvt_array_size * sizeof(int));
  for(i=0; i < c->mvt_array_nbelem; i++)
    c->mvt_array[i] = o->mvt_array[i]; 

  c->gfv = global_feat_vec_copy(o->gfv);
  return c;
}

void config_add_mvt(config *c, int mvt)
{
  if(c->mvt_array_size == c->mvt_array_nbelem){
    c->mvt_array_size = 2 * (c->mvt_array_size + 1);
    c->mvt_array = (int *)realloc(c->mvt_array, c->mvt_array_size * sizeof(int));
  }
  c->mvt_array[c->mvt_array_nbelem++] = mvt;
}

void config_print(FILE *f, config *c)
{
  /* word *b0 = NULL; */
  /* word *s0 = NULL; */
  if(c){
    if(!stack_is_empty(c->st))
    /*   s0  = stack_elt_n(c->st, 0); */
    /* b0  = word_buffer_b0(c->bf); */
    /* if(s0) { printf("s0 = "); word_print2(stdout, s0);} */
    /* if(b0) { printf("b0 = "); word_print2(stdout, b0);} */
    
    stack_print(f, c->st);
    fprintf(f, "\n");
    word_buffer_print_compact(f, c->bf);
  }
}

void config_connect_subtrees(config *c, int root_label)
{
  int i;

  if(stack_height(c->st) > 1){ /* stack contains other elements that dummy token */
    /* make all elements in the stack dependents of the dummy token with a root_label relation */
    for(i=1; i < stack_height(c->st); i++){
      depset_add(c->ds, c->st->array[0], root_label, c->st->array[i]);
    }
  }
}

int config_equal2(config *c1, config *c2)
{ 
  int i;

  fprintf(stdout, "pred score = %.4f\n", c1->score);
   fprintf(stdout, "mvt seq ");
  for(i=0; i < c1->mvt_array_nbelem; i++)
    fprintf(stdout, "%d ", c1->mvt_array[i]);
  fprintf(stdout, "\n");

  fprintf(stdout, "dep set ");
  depset_print(stdout, c1->ds);

  return depset_compare(c1->ds, c2->ds);
}

int config_equal(config *c1, config *c2)
{ 
  int i;
 
  /*  for(i=0; i < c2->mvt_array_nbelem; i++)
    fprintf(stdout, "%d ", c2->mvt_array[i]);
    fprintf(stdout, "\n");*/
  
  
  if(c1->mvt_array_nbelem != c2->mvt_array_nbelem)
    return 0;
  for(i=0; i < c1->mvt_array_nbelem; i++){
    if(c1->mvt_array[i] != c2->mvt_array[i])
      return 0;
  }
  return 1;
}
