#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#include "feat_model.h"
#include "feat_lib.h"
#include "feat_desc.h"
#include "feat_types.h"
#include "config2feat_vec.h"

void feat_model_free(feat_model *fm)
{
  int i;
  for(i=0; i < fm->nbelem; i++)
    feat_desc_free(fm->array[i]);
  free(fm->name);
  free(fm);
}

feat_model *feat_model_read(char *filename, int verbose)
{
  FILE *f = myfopen(filename, "r");
  feat_model *fm = feat_model_new(filename);
  int feature_number = 0;
  char buffer[1000]; /* ugly */
  char *feat_name;
  feat_lib *fl = feat_lib_build();
  simple_feat_desc *sfd;
  feat_desc *fd;
  
  while(fgets(buffer, 1000, f)){
    if(feof(f)) break;
    if((buffer[0] == '\n') || (buffer[0] == '#')) continue;
    if(verbose) fprintf(stderr, "%d", feature_number + 1);
    fd = feat_desc_new();
    feat_name = strtok(buffer, " \n");
    do{
      if(verbose) fprintf(stderr, "\t%s", feat_name);
      sfd = feat_lib_get_simple_feat_desc(fl, feat_name);
      if(sfd)
	feat_desc_add(fd, sfd);
    }while((feat_name = strtok(NULL, " \n")));
    if(verbose) fprintf(stderr, "\n");
    feat_model_add(fm, fd);
    feature_number++;
  }
  fclose(f);
  return fm;
}


void feat_model_compute_ranges(feat_model *fm, mcd *m, int mvt_nb)
{
  int i;
  feat_desc *fd;
  simple_feat_desc *sfd;
  int total = 0;
  int column;
  int mcd_representation;

  for(i = 0; i < fm->nbelem; i++){
    fd = fm->array[i];
    if(fd->nbelem > 1){
      /* fprintf(stderr, "cannot compute range of complex features\n"); */
    } 
    else{
      sfd = fd->array[0];
      if((sfd->type <= FEAT_TYPE_INT_16) && (sfd->type >= FEAT_TYPE_INT_1)){
	sfd->range = sfd->type - FEAT_TYPE_INT_0;
      }
      else if(sfd->type == FEAT_TYPE_TRANS){
	sfd->range = mvt_nb;
      }
      else{
	column = m->wf2col[sfd->type];
	mcd_representation = m->representation[column];
	if(mcd_representation == MCD_REPRESENTATION_VOCAB)
	  sfd->range = m->dico_array[column]->nbelem;
	else if(mcd_representation == MCD_REPRESENTATION_EMB)
	  sfd->range = m->word_emb_array[column]->dim;
      }
      total += sfd->range;
    }
  }
  fm->dim = total;
}

void catenate_int(char *string, int val)
{
  char s[10];
  sprintf(s, "%d", val);
  strcat(string, s);
}

int feat_model_get_feat_value_fann(feat_model *fm, config *c, int feat_nb)
{
  feat_desc *fd = fm->array[feat_nb];
  /* fm must be exclusively  composed of simple features */
  return fd->array[0]->fct(c);
}

int feat_model_get_feat_value_cff(feat_model *fm, config *c, dico *dico_features, int feat_nb, int mode)
{
  feat_desc *fd = fm->array[feat_nb];
  int i;
  int feat_val;

  /*  the name of the feature is built in fm->string and its value in the dictionnary (dico_features) is returned */
  fm->string[0] = '\0';
  for(i=0; i < fd->nbelem; i++){
    strcat(fm->string, fd->array[i]->name);
    feat_val = fd->array[i]->fct(c);
    catenate_int(fm->string, feat_val);
  }
 
  if(mode == LOOKUP_MODE){
    if(fm->string)
    /* printf("fmstring = %s\n", fm->string); */
    return dico_string2int(dico_features, fm->string);
  } 
  return dico_add(dico_features, fm->string);
}


feat_model *feat_model_new(char *name)
{
  feat_model *fm = (feat_model *)memalloc(sizeof(feat_model));
  fm->name = strdup(name);
  fm->nbelem = 0;
  fm->array = NULL;
  fm->dim = 0;
  return fm;
}

feat_desc *feat_model_add(feat_model *fm, feat_desc *fd)
{
  fm->nbelem++;
  fm->array = (feat_desc **)realloc(fm->array, fm->nbelem * sizeof(feat_desc *));
  fm->array[fm->nbelem - 1] = fd;
  return fd;
}
