#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include"util.h"
#include"movement_parser_arc_eager.h"

void movement_print(FILE *f, int mvt_code, dico *dico_labels){
  int mvt_type = movement_type(mvt_code);
  int mvt_label = movement_label(mvt_code);
  char *label;
  if(mvt_type == MVT_SHIFT)  {fprintf(f, "SHIFT\n"); return;}
  if(mvt_type == MVT_REDUCE) {fprintf(f, "REDUCE\n"); return;}
  if(mvt_type == MVT_ROOT)   {fprintf(f, "ROOT\n"); return;}
  if(mvt_type == MVT_EOS)    {fprintf(f, "EOS\n"); return;}
  if(mvt_type == MVT_RIGHT)   fprintf(f, "RIGHT");
  else fprintf(f, "LEFT");
  label = dico_int2string(dico_labels, mvt_label);
  fprintf(f, " %s\n", label);
}

int movement_type(int mvt)
{
  if(mvt == MVT_SHIFT)     return MVT_SHIFT;      /* 0  is the code of shift */
  if(mvt == MVT_REDUCE)    return MVT_REDUCE;     /* 1  is the code of reduce */
  if(mvt == MVT_ROOT)      return MVT_ROOT;       /* 2  is the code of root */
  if(mvt == MVT_EOS)       return MVT_EOS;        /* 3  is the code of root */
  if(mvt % 2 == 0)         return MVT_LEFT;       /* even movements are left movements */
  return MVT_RIGHT;                       /* odd movements are right movements */
}

int movement_label(int mvt)
{
  if(mvt == MVT_SHIFT) return -1;     /* 0  is the code of shift */ 
  if(mvt == MVT_REDUCE) return -1;     /* 1  is the code of reduce */ 
  if(mvt == MVT_ROOT) return -1;     /* 2  is the code of root */ 
  if(mvt == MVT_EOS) return -1;     /* 3  is the code of eos */ 
  if(mvt % 2 == 0)            /* even codes correspond to left movements */
    return mvt / 2 - 2;
  return (mvt - 1) / 2 - 2;   /* odd codes correspond to right movements */
}

int movement_eos(config *c, float score)
{
  if(stack_is_empty(config_get_stack(c))) return 1;
  
  /* word on the top of the stack is sent_seg */
  word_set_sent_seg(stack_top(config_get_stack(c)), 1);
  
  /* perform all pending reduce */
  while(movement_reduce(c,0));
  
  /* remove root from stack */
  stack_pop(config_get_stack(c));

  config_add_mvt(c, MVT_EOS);
  return 1;
}

int movement_left_arc(config *c, int label, float score)
{
   /* the dummy word cannot be a dependent */
  /* if(stack_height(config_get_stack(c)) < 2) return 0; */
  /* if(word_buffer_is_empty(config_get_buffer(c))) return 0; */
  /* word on top of the stack should not have a governor */
  if(word_get_gov(stack_top(config_get_stack(c))) != 0) return 0; 

  word *gov = word_buffer_b0(config_get_buffer(c));
  word *dep = stack_top(config_get_stack(c));
  int dist = (word_get_index(gov)) - (word_get_index(dep));

  /* printf("create left arc %d <- %d dist = %d\n", word_get_index(dep), word_get_index(gov), dist);  */
  
  /* create a new dependency */
  word_set_gov(dep, dist);
  word_set_label(dep, label);

  stack_pop(config_get_stack(c));
  config_add_mvt(c, movement_left_code(label)); 
  return 1;
}

int movement_right_arc(config *c, int label, float score)
{
  if(stack_is_empty(config_get_stack(c))) return 0;
  /* if(word_buffer_is_empty(config_get_buffer(c))) return 0; */
  
  word *gov = stack_top(config_get_stack(c));
  word *dep = word_buffer_b0(config_get_buffer(c));
  int dist = (word_get_index(gov)) - (word_get_index(dep));
  
  /* printf("create right arc %d -> %d dist = %d\n", word_get_index(gov), word_get_index(dep), dist);  */

  /* create a new dependency */
  word_set_gov(dep, dist);
  word_set_label(dep, label);
  
  stack_push(config_get_stack(c), word_buffer_b0(config_get_buffer(c)));
  word_buffer_move_right(config_get_buffer(c));
  
  config_add_mvt(c, movement_right_code(label));
  return 1;
}

int movement_shift(config *c, int stream, float score)
{
  if(word_buffer_is_empty(config_get_buffer(c))) return 0;
  stack_push(config_get_stack(c), word_buffer_b0(config_get_buffer(c)));
  word_buffer_move_right(config_get_buffer(c));
  config_add_mvt(c, MVT_SHIFT);
  return 1;
}

int movement_reduce(config *c, float score)
{
  if(stack_is_empty(config_get_stack(c))) return 0;
  /* word on top of stack must have a governor */
  if(word_get_gov(stack_top(config_get_stack(c))) == 0) return 0; 
  stack_pop(config_get_stack(c));
  config_add_mvt(c, MVT_REDUCE);
  return 1;
}

int movement_root(config *c, float score, int root_code)
{
  word *b0 = word_buffer_b0(config_get_buffer(c));
  word_set_gov(b0, 0);
  word_set_label(b0, root_code);
  config_add_mvt(c, MVT_ROOT);
  return 1;
}

