#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<getopt.h>
#include"context.h"
#include"movement_parser_arc_eager.h"
#include"feat_fct.h"
#include"config2feat_vec.h"
#include"feature_table.h"
#include"dico.h"

void print_word_buffer(config *c, dico *dico_labels)
{
  int i;
  word *dep;
  char *label;
  for(i=0; i < config_get_buffer(c)->nbelem; i++){
    dep = word_buffer_get_word_n(config_get_buffer(c), i);
    printf("%s\t", word_get_input(dep));
    printf("%d\t", word_get_gov(dep));
    label = (word_get_label(dep) == -1)? NULL : dico_int2string(dico_labels, word_get_label(dep));
    if(label != NULL)
      printf("%s\t", label) ;
    else
      printf("_\t");
    if((label != NULL) && !strcmp(label, "eos"))
      printf("1\n");
    else
      printf("0\n");
  }
}


void simple_decoder_parser_arc_eager(context *ctx)
{
  FILE *f = (ctx->input_filename)? myfopen(ctx->input_filename, "r") : stdin;
  feature_table *ft = feature_table_load(ctx->perc_model_filename, ctx->verbose);
  int root_label;
  int mvt_code;
  int mvt_type;
  int mvt_label;
  float max;
  feat_vec *fv = feat_vec_new(feature_types_nb);
  config *c = NULL;

  root_label = dico_string2int(ctx->dico_labels, ctx->root_label);
  if(root_label == -1) root_label = 0;
  
  c = config_initial_no_dummy_word(f, ctx->mcd_struct, 5);
  while(1){
    config2feat_vec_cff(ctx->features_model, c, ctx->d_perceptron_features, fv, LOOKUP_MODE);
    mvt_code = feature_table_argmax(fv, ft, &max);
    mvt_type = movement_type(mvt_code);
    mvt_label = movement_label(mvt_code);

    config_print(stdout, c);     
    movement_print(stdout, mvt_code, ctx->dico_labels);        

    if(mvt_type == MVT_LEFT)
      if(movement_left_arc(c, mvt_label, max)){
	if(word_buffer_is_last(config_get_buffer(c)))
	  break;
	continue;
      }
    
    if(mvt_type == MVT_RIGHT)
      if(movement_right_arc(c, mvt_label, max)){
	if(word_buffer_is_last(config_get_buffer(c)))
	  break;
	continue;
      }
    
    if(mvt_type == MVT_REDUCE)
      if(movement_reduce(c, max)){
	if(word_buffer_is_last(config_get_buffer(c)))
	  break;
	continue;
      }
    
    if(mvt_type == MVT_ROOT)
      if(movement_root(c, max, root_label)){
	if(word_buffer_is_last(config_get_buffer(c)))
	  break;
	continue;
      }
    
    /*    if(mvt_type == MVT_EOS)
      if(movement_eos(c, max)){
	if(word_buffer_is_last(config_get_buffer(c)))
	  break;
	continue;
      }
    */
    if(word_buffer_is_last(config_get_buffer(c)))
      break;
    
    movement_shift(c, 1, max);
    
  }
  
  print_word_buffer(c, ctx->dico_labels);
  
  /* config_free(c); */
  feat_vec_free(fv);
  feature_table_free(ft);
  if(ctx->input_filename)
    fclose(f);
}
