#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<getopt.h>
#include"context.h"
#include"movement_tagger.h"
#include"feat_fct.h"
#include"config2feat_vec.h"
#include"feature_table.h"
#include"dico.h"

void add_signature_to_words_in_queue(queue *bf, form2pos *f2p)
{
  int i;
  word *w;

  for(i=0; i < queue_nbelem(bf); i++){
    w = queue_elt_n(bf, i);

    w->signature = form2pos_get_signature(f2p, w->form);
  }
}

void add_signature_to_words_in_word_buffer(word_buffer *bf, form2pos *f2p)
{
  int i;
  word *w;

  for(i = word_buffer_get_nbelem(bf) - 1; i >=0  ; i--){
    w = word_buffer_get_word_n(bf, i);
    if(word_get_signature(w) != -1) break;
    w->signature = form2pos_get_signature(f2p, w->form);
  }
}

void simple_decoder_buffer(context *ctx)
{
  dico *dico_pos = dico_vec_get_dico(ctx->vocabs, (char *)"POS");
  feature_table *ft =  feature_table_load(ctx->perc_model_filename, ctx->verbose);
  int postag;
  feat_vec *fv = feat_vec_new(feature_types_nb);
  float max;
  int i;
  word *w = NULL;
  FILE *f = (ctx->input_filename)? myfopen(ctx->input_filename, "r") : stdin;
  config *c = config_initial_no_dummy_word(f, ctx->mcd_struct, 0);

  /* read a sentence and put it in the buffer */
  while(word_buffer_read_sentence(c->bf)){
    /* queue_remove(c->bf); */
    /* get rid of dummy token */
    if(ctx->f2p)
      add_signature_to_words_in_word_buffer(c->bf, ctx->f2p);
    while(!config_is_terminal(c)){
      config2feat_vec_cff(ctx->features_model, c, ctx->d_perceptron_features, fv, LOOKUP_MODE);
      
      postag = feature_table_argmax(fv, ft, &max);
      if(postag != -1)
	movement_tagger(c, postag, max, 0);
    }
    /* config_print(stdout, c);   */

    for(i = stack_nbelem(c->st)-1; i >= 0 ; i--){
      w = stack_elt_n(c->st, i);
      printf("%s\t%s\n", w->input, dico_int2string(dico_pos, word_get_pos(w)));
    }
    /* printf("\n"); */

    /* config_free(c);  */
    c = config_initial_no_dummy_word(f, ctx->mcd_struct, 0);
  }
  if(ctx->input_filename)
    fclose(f);
}


void simple_decoder_stream(context *ctx)
{
  config *c;
  feat_vec *fv = feat_vec_new(feature_types_nb);
  FILE *f = (ctx->input_filename)? myfopen(ctx->input_filename, "r") : stdin;
  feature_table *ft =  feature_table_load(ctx->perc_model_filename, ctx->verbose);
  int postag;
  float max;
  word *w;
  dico *dico_pos = dico_vec_get_dico(ctx->vocabs, (char *)"POS");
  int res;
  c = config_initial_no_dummy_word(f, ctx->mcd_struct, 5); 

  while(1){
    if(ctx->f2p)
      add_signature_to_words_in_word_buffer(c->bf, ctx->f2p);
    /* config_print(stdout, c); */
    config2feat_vec_cff(ctx->features_model, c, ctx->d_perceptron_features, fv, LOOKUP_MODE);
    
    /* feat_vec_print(stdout, fv); */
    postag = feature_table_argmax(fv, ft, &max);
    /* printf("postag = %d\n", postag); */

    w = word_buffer_b0(c->bf);
    printf("%s\t%s\n", w->input, dico_int2string(dico_pos, postag));

    res = movement_tagger(c, postag, max, 1);

    /* printf(" current index = %d nb elem = %d\n", c->bf->current_index, c->bf->nbelem);  */

    if(res == 0) break;
  }

  /* config_print(stdout, c);  */
  /* config_free(c); */
}


void simple_decoder_tagger(context *ctx)
{
  if(ctx->stream_mode)
    simple_decoder_stream(ctx);
  else
    simple_decoder_buffer(ctx);
}

